#!/usr/bin/env python3
# ------------------------------ CODE CELL SETUP -------------------------------
__IB_FLAG__ = True
__IMPORT_FILTER__ = globals().get('IMPORT_FILTER', None)
__PLOTS__ = []
__LABEL__ = None

if __IMPORT_FILTER__ is not None:
    regex, blacklist = __IMPORT_FILTER__
    print(f'set import filter: regex=r"{regex}", blacklist={blacklist}')

try:
    # If matplotlib is available in the test environment, it is set to headless mode
    # and all plots are dumped to disk rather than being displayed.
    import matplotlib as _mpl
    _mpl.use('Agg')

    from functools import wraps
    from pathlib import Path

    import matplotlib.pyplot as _plt

    from autograde.util import snake_case

    __show = _plt.show
    __save = _plt.savefig

    @wraps(__save)
    def _save(*args, **kwargs):
        __save(*args, **kwargs)
        _plt.close()

    @wraps(__show)
    def _show(*_, **__):
        if _plt.gcf().get_axes():
            root = Path('figures')
            root.mkdir(exist_ok=True)
            path = root / snake_case(f'fig_cell_{__LABEL__}_{len(__PLOTS__) + 1}')
            __PLOTS__.append(path)

            print(f'save figure at {path}')
            _save(path)

    _plt.savefig = _save
    _plt.show = _show

except ImportError:
    pass

auto_save_figure = globals().get('_show', lambda *args, **kwargs: None)

# EXECUTED IN 0.477s
# STDOUT
#     set import filter: regex=r"re.compile('autograde')", blacklist=True

# -------------------------------- CODE CELL 1 ---------------------------------
team_members = [
    {
        'first_name': 'Alice',
        'last_name': 'Foo',
        'student_id': 12345
    },
    {
        'first_name': 'Bob',
        'last_name': 'Bar',
        'student_id': 54321
    }
]

# EXECUTED IN 0.00111s

# ----------------------------- CODE CELL 1 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.00652s

# -------------------------------- CODE CELL 2 ---------------------------------
import sys
import time

import matplotlib.pyplot as plt
import pandas as pd

# EXECUTED IN 1.4s

# ----------------------------- CODE CELL 2 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.0012s

# -------------------------------- CODE CELL 3 ---------------------------------
SOME_CONSTANT = 42

# EXECUTED IN 0.00096s

# ----------------------------- CODE CELL 3 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.001s

# -------------------------------- CODE CELL 4 ---------------------------------
df = pd.DataFrame([[1, 1], [2, 4], [3, 9], [4, 16], [5, 25]])
df.plot()

# EXECUTED IN 0.626s

# ----------------------------- CODE CELL 4 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.563s
# STDOUT
#     save figure at figures/fig_cell_4_clean_1

# -------------------------------- CODE CELL 5 ---------------------------------
from IPython.display import display

display(df)

# EXECUTED IN 0.1s
# STDOUT
#     0   1
#     0  1   1
#     1  2   4
#     2  3   9
#     3  4  16
#     4  5  25

# ----------------------------- CODE CELL 5 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.00114s

# -------------------------------- CODE CELL 6 ---------------------------------
with open('foo.txt', mode='rt') as f:
    data = f.read()

print(data[:len(data) // 2])
print(data[len(data) // 2:], file=sys.stderr)

# EXECUTED IN 0.00136s
# STDOUT
#     this file gets
# STDERR
#     read by notebook

# ----------------------------- CODE CELL 6 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.000894s

# -------------------------------- CODE CELL 7 ---------------------------------
with open('bar.txt', mode='a') as f:
    f.write('>> modification by notebook <<')

# EXECUTED IN 0.000959s

# ----------------------------- CODE CELL 7 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.000892s

# -------------------------------- CODE CELL 8 ---------------------------------
with open('fnord.txt', mode='wt') as f:
    f.write('let\'s leave some artifacts in the system')

# EXECUTED IN 0.00115s

# ----------------------------- CODE CELL 8 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.000714s

# -------------------------------- CODE CELL 9 ---------------------------------
for e in [2, 3]:
    plt.plot(*zip(*((v, v ** e) for v in range(-10, 10))))
    plt.show()

# EXECUTED IN 1.65s
# STDOUT
#     save figure at figures/fig_cell_9_1
#     save figure at figures/fig_cell_9_2

# ----------------------------- CODE CELL 9 CLEAN ------------------------------
auto_save_figure()

# EXECUTED IN 0.00474s

# -------------------------------- CODE CELL 10 --------------------------------
plt.plot(*zip(*((v, v ** 5 + (v - 5) ** 3 - (3 * v) ** 2) for v in range(-20, 20))))
plt.savefig('plot.png')

# EXECUTED IN 0.857s

# ----------------------------- CODE CELL 10 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.00621s

# -------------------------------- CODE CELL 11 --------------------------------
def square(x: float) -> float:
    return x ** 2


def cube(x: float) -> float:
    return x ** 3


abs_cube = cube


def fail():
    raise ValueError('no chance, this function crashes so badly')


def sleep(x):
    time.sleep(x)

# EXECUTED IN 0.00129s

# ----------------------------- CODE CELL 11 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.00104s

# -------------------------------- CODE CELL 12 --------------------------------
sleep(.1)

# EXECUTED IN 0.101s

# ----------------------------- CODE CELL 12 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.00157s

# -------------------------------- CODE CELL 13 --------------------------------
import autograde

print(autograde.__version__)

# EXECUTED IN 0.00146s
# STDERR
#     Traceback (most recent call last):
#       File "/opt/conda/lib/python3.9/site-packages/autograde/notebook_executor.py", line 133, in exec_notebook
#         shadow_stack.enter_context(shadow_exec(code_cell, state))
#       File "/opt/conda/lib/python3.9/contextlib.py", line 429, in enter_context
#         result = _cm_type.__enter__(cm)
#       File "/opt/conda/lib/python3.9/contextlib.py", line 117, in __enter__
#         return next(self.gen)
#       File "/opt/conda/lib/python3.9/site-packages/autograde/notebook_executor.py", line 61, in shadow_exec
#         exec(compiled_source, *args, **kwargs)
#       File "/tmp/tmpiq3u368r", line 1, in <module>
#       File "/opt/conda/lib/python3.9/unittest/mock.py", line 1093, in __call__
#         return self._mock_call(*args, **kwargs)
#       File "/opt/conda/lib/python3.9/unittest/mock.py", line 1097, in _mock_call
#         return self._execute_mock_call(*args, **kwargs)
#       File "/opt/conda/lib/python3.9/unittest/mock.py", line 1158, in _execute_mock_call
#         result = effect(*args, **kwargs)
#       File "/opt/conda/lib/python3.9/site-packages/autograde/util.py", line 277, in guard
#         raise ImportError(f'usage of "\'{target}\'" is not permitted')
#     ImportError: usage of "'autograde'" is not permitted

# ----------------------------- CODE CELL 13 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.000893s

# -------------------------------- CODE CELL 14 --------------------------------
def illegal_import():
    exec('import autograde as ag')

# EXECUTED IN 0.000776s

# ----------------------------- CODE CELL 14 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.000721s

# -------------------------------- CODE CELL 15 --------------------------------
assert False, 'this cell will crash under any circumstances ...'

# EXECUTED IN 0.000835s
# STDERR
#     Traceback (most recent call last):
#       File "/opt/conda/lib/python3.9/site-packages/autograde/notebook_executor.py", line 133, in exec_notebook
#         shadow_stack.enter_context(shadow_exec(code_cell, state))
#       File "/opt/conda/lib/python3.9/contextlib.py", line 429, in enter_context
#         result = _cm_type.__enter__(cm)
#       File "/opt/conda/lib/python3.9/contextlib.py", line 117, in __enter__
#         return next(self.gen)
#       File "/opt/conda/lib/python3.9/site-packages/autograde/notebook_executor.py", line 61, in shadow_exec
#         exec(compiled_source, *args, **kwargs)
#       File "/tmp/tmpwr9vhbgf", line 1, in <module>
#     AssertionError: this cell will crash under any circumstances ...

# ----------------------------- CODE CELL 15 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.000885s

# -------------------------------- CODE CELL 16 --------------------------------
print('... but this cell is not affected :^)')

# EXECUTED IN 0.00107s
# STDOUT
#     ... but this cell is not affected :^)

# ----------------------------- CODE CELL 16 CLEAN -----------------------------
auto_save_figure()

# EXECUTED IN 0.000984s

# ----------------------------- CODE CELL TEARDOWN -----------------------------
__IA_FLAG__ = True

# EXECUTED IN 0.000385s

